/*
* OpenGL - 3D Vision Wrapper
* Copyright (c) Helifax 2015
*/

#ifndef __SHADER__MANAGER_H__
#define __SHADER__MANAGER_H__

#include <string>
#include <vector>
#include <stdlib.h> 
#include <gl_custom.h>

// Stucture that will hold the existing shaders + program that they are attached to
typedef struct
{
	GLuint m_programId;
	GLuint m_shaderId;
	DWORD m_CRC32;
	GLuint m_shaderType;
	std::string m_shaderSourceCode;
} EXISTING_SHADER_T;

// Structure that will hold the custom shaders
typedef struct
{
	bool m_isApplied;
	DWORD m_CRC32;
	GLuint m_shaderType;
	std::string m_shaderSourceCode;
} CUSTOM_SHADER_T;

// Strictire holding the custom shader Pair read from disk (Ps+Vs).
typedef struct
{
	bool m_isApplied;
	DWORD m_CRC32Vertex;
	DWORD m_CRC32Pixel;
	GLuint m_shaderType;
	std::string m_shaderSourceCode;
} CUSTOM_SHADER_PAIR_T;

// Custom values sent to m_programId Shader Program.
typedef struct  
{
	GLuint m_programId;
	float x;
	float y;
	float z;
	float w;
}CUSTOM_SHADER_VALUES_T;

// Uniforms used for 3D and shader manipulation
typedef struct
{
	// Supported uniforms
	GLint location_eye;
	GLint location_eye_separation;
	GLint location_convergence;
	GLint location_vertexEnabled;
	GLint location_pixelEnabled;
	GLint location_screenInfo;
	GLint location_custom_params;

	// Our current active program
	GLuint currentProgram;

}SHADER_3D_UNIFORMS_T;

// Defines on how many uniforms we support
#define MAX_UNIFORM_POINTS					4
#define MAX_VERTEX_INJECTION_POINTS			10

// MAIN CLASS
// Name of functions are self-explanatory.
class ShaderManager
{
public:
	static ShaderManager* getInstance()
	{
		if (instance == 0)
		{
			instance = new ShaderManager;
		}
		return instance;
	}

	//default dtor nothing to do 
	~ShaderManager() { ; }

	void ReadConfigFile(void);

	void LoadCustomShaders(void);

	std::string getShaderSource(GLuint shaderId);

	bool isShaderType(GLuint _shaderID, GLenum _shaderType);

	std::string injectVertexModifications(std::string _inputSource, GLuint programId);

	std::string injectFragmentModification(std::string _inputSource, GLuint programId);

	void applyShaderSource(GLuint _shaderID, std::string _shaderSource, const GLint *length);

	// Set the shader pair as applied
	bool ApplyExceptionShaders(std::string &newShaderSource, GLenum shaderType, DWORD CRC32);

	// Apply custom parameters
	void ApplyCustomShaderParams(SHADER_3D_UNIFORMS_T &uniforms);
	void ApplyCustomShaderParamsARB(SHADER_3D_UNIFORMS_T &uniforms);

	// Get the custom parameters
	BOOL GetCustomShaderParam(GLuint programId, CUSTOM_SHADER_VALUES_T *pCustomShaderParam);

	// Get the stored Shader Source from program and type
	std::string GetShaderSourceCode(GLuint programId, GLuint startIndex, GLenum shaderType);

	//ShaderInjection toggle
	inline void SetVertexInjectionState(bool state)
	{
		m_enableStereoVertexInjection = state;
	}

	inline void SetFgramentInjectionState(bool state)
	{
		m_enableStereoFragmentInjection = state;
	}

	inline bool GetVertexInjectionState(void)
	{
		return m_enableStereoVertexInjection;
	}

	inline bool GetUnity5State(void)
	{
		return m_enableUnity5Mode;
	}

	inline bool GetProgInCRCState(void)
	{
		return m_embedProgInCRC;
	}

	inline bool GetStereoFailureInfoState(void)
	{
		return m_vertexAutoStereoInjectionInfo;
	}

	inline bool GetFragmentInjectionState(void)
	{
		return m_enableStereoFragmentInjection;
	}

	// Uniforms
	inline std::string GetUniformPoint(size_t number)
	{
		std::string res = "NONE";
		if (number < MAX_UNIFORM_POINTS)
			res = m_UniformPoint[number];
		return res;
	}

	//Get the injection point where we will inject the Vertex source code
	inline std::string GetVertexInjectionPoint(size_t number)
	{
		std::string res = "NONE";
		if (number < MAX_VERTEX_INJECTION_POINTS)
			res = m_vertexInjectionPoint[number];
		return res;
	}
	
	//Get the string that we will inject the Vertex source code
	inline std::string GetVertexStereoInjectString(size_t number)
	{
		std::string res = "";
		if (number < MAX_VERTEX_INJECTION_POINTS)
			res = m_vertexStereoInjectionString[number];
		return res;
	}

	// Are we inserting or replacing the source code?
	inline bool GetVertexIsInsert(size_t number)
	{
		if (number < MAX_VERTEX_INJECTION_POINTS)
			return m_vertexIsInsert[number];
		else
			return true;
	}

	//Get the injection point where we will inject the Pixel source code
	inline std::string GetPixelInjectionPoint(size_t number)
	{
		std::string res = "NONE";
		if (number < MAX_VERTEX_INJECTION_POINTS)
			res = m_pixelInjectionPoint[number];
		return res;
	}
	
	//Get the string that we will inject the Pixel source code
	inline std::string GetPixelStereoInjectString(size_t number)
	{
		std::string res = "";
		if (number < MAX_VERTEX_INJECTION_POINTS)
			res = m_pixelStereoInjectionString[number];
		return res;
	}

	// Are we inserting or replacing the source code?
	inline bool GetPixelIsInsert(size_t number)
	{
		if (number < MAX_VERTEX_INJECTION_POINTS)
			return m_pixelIsInsert[number];
		else
			return true;
	}

	// Did we already applied the custom shaders?
	inline bool areShadersApplied( int _shaderPairIndex)
	{
		return m_exceptShaders[_shaderPairIndex].m_isApplied;
	}

	//Exisint shaders + source at creation
	inline void addExistingShaderInfo(EXISTING_SHADER_T _shaders)
	{
		m_existingShaders.push_back(_shaders);
	}

	inline std::vector<EXISTING_SHADER_T> GetExistingShaders(void)
	{
		return m_existingShaders;
	}

	// Apply the custom shader pair (Vs+Ps)
	void ApplyShaderPair(void);

	// Set the debug shader pair as applied
	bool ApplyDebugExceptionShaders(void);

	// The program Id is found in the existing shader list??
	// Returns -1 if not found or the index position
	int ShaderProgramFound(int progId, GLuint shaderType);

	// get the start exception shader
	inline int GetExceptionShaderStart()
	{
		return m_shaderStart;
	}

	// get the end exception shader
	inline int GetExceptionShaderEnd()
	{
		return m_shaderEnd;
	}

	// Are vertex shaders exceptions enabled ???
	inline bool VertexShadersExceptionsEnabled()
	{
		return m_exceptShadersEnabled;
	}

	// Do we try to disable the current vertex shader ?
	inline bool VertexShaderExceptionsDisableShader()
	{
		return m_disableExceptShaders;
	}

	inline bool DisableCurrentShader()
	{
		return m_disableCurrentShader;
	}

	inline std::string GetVertexDisableString()
	{
		return m_vertexDisableString;
	}

	inline std::string GetPixelDisableString()
	{
		return m_pixelDisableString;
	}

	inline bool IsShaderSwapContinous()
	{
		return m_swapIsContinous;
	}

	inline bool IsShaderPairSwapContinous()
	{
		return m_swapPairIsContinous;
	}

	
private:

	//default ctor 
	ShaderManager();
	static ShaderManager * instance;
	ShaderManager(const ShaderManager&);
	ShaderManager& operator= (const ShaderManager&);
	// our shader list containing the vertex + fragment pair and new shader source
	std::vector<CUSTOM_SHADER_T> m_exceptShaders;
	std::vector<CUSTOM_SHADER_PAIR_T> m_exceptPairShaders;

	bool m_enableStereoVertexInjection;
	bool m_enableStereoFragmentInjection;
	bool m_vertexAutoStereoInjectionInfo;

	// Unity 5
	bool m_enableUnity5Mode;

	// ProgramId in CRC embed?
	bool m_embedProgInCRC;

	// Uniforms
	std::string m_UniformPoint[MAX_UNIFORM_POINTS];

	// Vertex
	std::string m_vertexInjectionPoint[MAX_VERTEX_INJECTION_POINTS];
	std::string m_vertexStereoInjectionString[MAX_VERTEX_INJECTION_POINTS];
	bool m_vertexIsInsert[MAX_VERTEX_INJECTION_POINTS];

	// Pixel
	std::string m_pixelInjectionPoint[MAX_VERTEX_INJECTION_POINTS];
	std::string m_pixelStereoInjectionString[MAX_VERTEX_INJECTION_POINTS];
	bool m_pixelIsInsert[MAX_VERTEX_INJECTION_POINTS];

	// Stop after one shader have been found and swapped? or look for more identical shaders and swap them as well
	bool m_swapIsContinous;

	// As above but applies to PAIR shaders!!! (CRC of Vertex + Pixel MUST MATCH)
	bool m_swapPairIsContinous;
	
	// Shader Exceptions. Debug Only
	bool m_exceptShadersEnabled;
	bool m_disableExceptShaders;
	bool m_disableCurrentShader;
	int m_shaderStart;
	int m_shaderEnd;

	std::string  m_vertexDisableString;
	std::string  m_pixelDisableString;

std::vector<EXISTING_SHADER_T> m_existingShaders;
std::vector<CUSTOM_SHADER_VALUES_T> m_customShaderValues;
};

// Public Functions
void CheckCompileState(GLuint newShaderId, GLuint crc32);
void CheckLinkState(GLuint programId);
void DeleteAndDetachShaders(GLuint progId, GLuint shaderId);
void ExportShader(std::string _shaderType, GLuint _program, DWORD crc32, std::string _shaderSource);
void ExportCompiledShader(std::string _shaderSource);

SHADER_3D_UNIFORMS_T GetCurrent3DUniforms(void);
void SetCurrent3DUniforms(SHADER_3D_UNIFORMS_T uniforms);

#endif

